@@ -1,6 +1,7 @@
 # SYS imports
 import os
 import sys
+from trace import Trace
 
 # Append subdirectories to sys.path
 ppath = os.path.abspath(os.path.join(os.path.dirname(__file__),os.pardir))
@@ -13,15 +14,15 @@ sys.path.append(fpath)
 print(sys.path)
 
 # Old imports for API Calls
-import requests
-from requests.exceptions import HTTPError
+# import requests
+# from requests.exceptions import HTTPError
 
 # General Imports
 import pickle
 import time
 import logging
 import json
-import yaml
+# import yaml
 from pathlib import Path
 
 # New imports to optimize API-Calls
@@ -35,6 +36,11 @@ import urllib.request
 # Import from lupulib
 import lupulib
 import lupulib.devices
+from lupulib.devices.binary_sensor import LupusecBinarySensor
+from lupulib.devices.sensor import LupusecSensor
+from lupulib.devices.switch import LupusecSwitch
+from lupulib.devices.thermal_switch import LupusecThemalSwitch
+from lupulib.devices.updown_switch import LupusecUpDownSwitch
 import lupulib.constants as CONST
 import lupulib.exceptions
 # from lupulib.exceptions import LupusecParseError, LupusecRequestError, LupusecResponseError
@@ -118,17 +124,66 @@ class LupusecAPI:
                     return {}
 
                 # Get Response Body
-                content = await resp.json()
+                # content = await resp.json()
+                content = await resp.text()
 
                 # ToDo: check for empty body, size = 0
                 content = content.replace(chr(245), "")
                 content = content.replace("\t", "")
-                _LOGGER.debug("Data Type of Response: =%s", type(content))
+                clean_content = json.loads(content)
+                _LOGGER.debug("Data Type of Response: =%s", type(clean_content))
                 end_time = time.time()
                 _LOGGER.debug(f"Endtime: {end_time}")   
                 _LOGGER.debug(f"Duration: {end_time - start_time} seconds") 
                 _LOGGER.debug("API-Call finished.")              
-                return content
+                return clean_content
+
+        except aiohttp.client_exceptions.ClientConnectorError:
+            _LOGGER.error("Cannot connect to: ", url)
+            return {}
+
+        except aiohttp.ContentTypeError:
+            _LOGGER.error("JSON decode failed")
+            return {}
+
+
+    async def _async_api_post(ip, client, action_url, params) -> Dict:
+        """Generic sync method to call the Lupusec API"""
+        # Generate complete URL from Constants.py
+        url = f'{CONST.URL_HTTP}{ip}{CONST.URL_ACTION}{action_url}'
+        _LOGGER.debug("_async_api_post() called: URL=%s", url)
+        start_time = time.time()
+        _LOGGER.debug(f"Starttime: {start_time}")
+
+        try:
+            async with client.post(url, data=params) as resp:
+                _LOGGER.debug("Response_Status=%s", resp.status)
+                _LOGGER.debug("Content_Type=%s", resp.headers["content-type"])
+
+                # check for Response Status other than 200
+                if resp.status != 200:
+                    _LOGGER.error(f"ERROR: Response status = {resp.status}")
+                    return {}
+
+                # check for non-JSON Response Headers   
+                if not resp.headers["content-type"].strip().startswith("application/json"):
+                    _LOGGER.error(f"ERROR: Content Type is not JSON = {resp.headers['content-type']}")
+                    return {}
+
+                # Get Response Body
+                # content = await resp.json()
+                content = await resp.text()
+
+                # ToDo: check for empty body, size = 0
+                content = content.replace(chr(245), "")
+                content = content.replace("\t", "")
+                clean_content = json.loads(content)
+                _LOGGER.debug("Data Type of Response: =%s", type(clean_content))
+                end_time = time.time()
+                _LOGGER.debug(f"Endtime: {end_time}")   
+                _LOGGER.debug(f"Duration: {end_time - start_time} seconds") 
+                _LOGGER.debug("API-Call finished.")              
+                return clean_content
 
         except aiohttp.client_exceptions.ClientConnectorError:
             _LOGGER.error("Cannot connect to: ", url)
@@ -168,10 +223,29 @@ class LupusecAPI:
         _LOGGER.debug("__init__.py.async_get_system() finished.")            
 
 
-    def _request_post(self, action, params={}):
-        return self.session.post(
-            self.api_url + action, data=params, headers=self.headers
-        )
+    async def async_set_mode(self, mode) -> None:
+        """Async method to set alarm mode."""
+        _LOGGER.debug("__init__.py.async_set_mode() called: ")
+        _LOGGER.info("...mode: ", str(mode))
+
+        params = {"mode": mode, "area": 1}
+
+         # Set Alarm Mode
+        async with aiohttp.ClientSession(auth=self._auth) as client:
+            tasks = []
+
+            # INFO_REQUEST
+            _LOGGER.debug("__init__.py.async_set_mode(): REQUEST=%s", CONST.SET_ALARM_REQUEST)
+            tasks.append(asyncio.ensure_future(LupusecAPI._async_api_post(self._ip_address, client, 
+                CONST.SET_ALARM_REQUEST, params)))
+
+            # Print response list
+            _LOGGER.debug("await asyncio.gather(*tasks)...")
+            response_list = await asyncio.gather(*tasks)
+            _LOGGER.debug("done. check content in response_list...")
+            for content in response_list:
+                print(content)              
+        _LOGGER.debug("__init__.py.async_set_mode() finished.")
 
 
     def get_power_switches(self):
@@ -239,23 +313,25 @@ class LupusecAPI:
             response_list = await asyncio.gather(*tasks)
             _LOGGER.debug("done. check content in response_list...")
             for content in response_list:
-                print(content)
                 # Retreive Device Liste from Response
                 if CONST.DEVICE_LIST_HEADER in content:
                     device_content = content[CONST.DEVICE_LIST_HEADER]
+                    print("Number of devices=", len(device_content))                    
                     api_devices = []
                     for device in device_content:
-                        if "openClose" in device:
-                                device["status"] = device["openClose"]
-                                device.pop("openClose")
-                        device["device_id"] = device[self.api_device_id]
-                        device.pop("cond")
-                        device.pop(self.api_device_id)
-                        if device["status"] == "{WEB_MSG_DC_OPEN}":
-                            print("yes is open " + device["name"])
-                            device["status"] = 1
-                        if device["status"] == "{WEB_MSG_DC_CLOSE}" or device["status"] == "0":
-                            device["status"] = "Geschlossen"
+                        #if "openClose" in device:
+                        #        device["status"] = device["openClose"]
+                        #        device.pop("openClose")
+                        #device["device_id"] = device[self.api_device_id]
+                        #device.pop("cond")
+                        #device.pop(self.api_device_id)
+                        #if device["status"] == "{WEB_MSG_DC_OPEN}":
+                        #    print("yes is open " + device["name"])
+                        #    device["status"] = 1
+                        #if device["status"] == "{WEB_MSG_DC_CLOSE}" or device["status"] == "0":
+                        #    device["status"] = "Geschlossen"
+                        print("sid: ", device["sid"], ", name: ", device["name"], 
+                            ", type: ", device["type"], ", status: ", device["status"])
                         api_devices.append(device)
                 self._apiDevices = api_devices
 
@@ -280,7 +356,7 @@ class LupusecAPI:
         panel["type"] = CONST.ALARM_TYPE
         panel["name"] = CONST.ALARM_NAME
 
-        # history = self.get_history()
+        history = self.get_history()
 
         if self.model == 1:
             for histrow in history:
@@ -312,7 +388,7 @@ class LupusecAPI:
         self.get_devices(refresh=True)
 
 
-    async def get_devices(self, refresh=True, generic_type=None) -> Dict:
+    async def get_devices(self, refresh=True) -> Dict:
         """Get all devices from Lupusec."""
         _LOGGER.debug("get_devices() called: ")
         # Make API-call only, if device list is empty or needs refresh
@@ -332,7 +408,8 @@ class LupusecAPI:
             _LOGGER.debug("...API-Call: response received...")
             _LOGGER.debug("...iterate over all devices in responseObject:")
             for deviceJson in responseObject:
-                print(deviceJson)
+                print("sid: ", deviceJson["sid"], ", name: ", deviceJson["name"], 
+                            ", type: ", deviceJson["type"], ", status: ", deviceJson["status"])
                 # Attempt to reuse an existing device
                 device = self._devices.get(deviceJson["name"])
                 _LOGGER.debug("...device: " + deviceJson["name"])
@@ -342,7 +419,7 @@ class LupusecAPI:
                     device.update(deviceJson)
                 else:
                     _LOGGER.debug("...newDevice found: " + deviceJson["name"])
-                    device = self.newDevice(deviceJson, self)
+                    device = newDevice(deviceJson, self)
 
                     if not device:
                         _LOGGER.info("Device is unknown")
@@ -351,47 +428,16 @@ class LupusecAPI:
                     self._devices[device.device_id] = device
 
             # We will be treating the Lupusec panel itself as an armable device.
-            panelJson = self.get_panel()
-            _LOGGER.debug("Get the panel in get_devices: %s", panelJson)
+            #panelJson = self.get_panel()
+            #_LOGGER.debug("Get the panel in get_devices: %s", panelJson)
+            #self._panel.update(panelJson)
 
-            self._panel.update(panelJson)
-
-            alarmDevice = self._devices.get("0")
-
-            if alarmDevice:
-                alarmDevice.update(panelJson)
-            else:
-                alarmDevice = devices.LupusecAlarm.create_alarm(panelJson, self)
-                self._devices["0"] = alarmDevice
-
-            # Now we will handle the power switches
-            if self.model == 1:
-                switches = self.get_power_switches()
-                _LOGGER.debug("Get active the power switches in get_devices: %s", switches)
-
-                for deviceJson in switches:
-                    # Attempt to reuse an existing device
-                    device = self._devices.get(deviceJson["name"])
-
-                    # No existing device, create a new one
-                    if device:
-                        device.update(deviceJson)
-                    else:
-                        device = self.newDevice(deviceJson, self)
-                        if not device:
-                            _LOGGER.info("Device is unknown")
-                            continue
-                        self._devices[device.device_id] = device
-
-            elif self.model == 2:
-                _LOGGER.debug("Power switches for XT2 not implemented")
-
-        if generic_type:
-            devices = []
-            for device in self._devices.values():
-                if device.type is not None and device.type in generic_type[0]:
-                    devices.append(device)
-            return devices
+            # alarmDevice = self._devices.get("0")
+            #if alarmDevice:
+            #    alarmDevice.update(panelJson)
+            #else:
+            #    alarmDevice = devices.LupusecAlarm.create_alarm(panelJson, self)
+            #    self._devices["0"] = alarmDevice
 
         return list(self._devices.values())
 
@@ -421,18 +467,6 @@ class LupusecAPI:
         return self.get_device(CONST.ALARM_DEVICE_ID, refresh)
 
 
-    def set_mode(self, mode):
-        if self.model == 1:
-            params = {
-                "mode": mode,
-            }
-        elif self.model == 2:
-            params = {"mode": mode, "area": 1}
-        r = self._request_post("panelCondPost", params)
-        responseJson = self.clean_json(r.text)
-        return responseJson
-
-
     def clean_json(textdata):
             # textdata = textdata.replace(chr(245), "")
         return textdata
@@ -440,18 +474,26 @@ class LupusecAPI:
 
 def newDevice(deviceJson, lupusec):
     """Create new device object for the given type."""
-    _LOGGER.debug("newDevice() called: name=" + deviceJson["name"])
     type_tag = deviceJson.get("type")
 
     if not type_tag:
         _LOGGER.info("Device has no type")
 
-    if type_tag in CONST.TYPE_OPENING:
+    if type_tag in CONST.TYPES_BIN_SENSOR:
+        _LOGGER.debug("newDevice(): name: " + deviceJson["name"] + "; type: " + str(type_tag) + " = BIN_SENSOR")
         return LupusecBinarySensor(deviceJson, lupusec)
-    elif type_tag in CONST.TYPE_SENSOR:
-        return LupusecBinarySensor(deviceJson, lupusec)
-    elif type_tag in CONST.TYPE_SWITCH:
+    elif type_tag in CONST.TYPES_SENSOR:
+        _LOGGER.debug("newDevice(): name=" + deviceJson["name"] + "; type=" + str(type_tag) + " = SENSOR")        
+        return LupusecSensor(deviceJson, lupusec)
+    elif type_tag in CONST.TYPES_SWITCH:
+        _LOGGER.debug("newDevice(): name=" + deviceJson["name"] + "; type=" + str(type_tag) + "= SWITCH")        
         return LupusecSwitch(deviceJson, lupusec)
+    elif type_tag in CONST.TYPES_UPDOWN_SWITCH:
+        _LOGGER.debug("newDevice(): name=" + deviceJson["name"] + "; type=" + str(type_tag) + "= UPDOWN_SWITCH")        
+        return LupusecThemalSwitch(deviceJson, lupusec)
+    elif type_tag in CONST.TYPES_THERMAL_SWITCH:
+        _LOGGER.debug("newDevice(): name=" + deviceJson["name"] + "; type=" + str(type_tag) + "= THERMAL_SWITCH")        
+        return LupusecUpDownSwitch(deviceJson, lupusec)                
     else:
         _LOGGER.info("Device is not known")
     return None
