@@ -4,7 +4,7 @@
 # -*- coding: utf-8 -*-
 
 # Project Information
-VERSION = "0.3.20"
+VERSION = "0.3.36"
 PROJECT_PACKAGE_NAME = "lupulib"
 PROJECT_LICENSE = "MIT"
 PROJECT_URL = "http://www.github.com/majuss/lupulib"
@@ -20,13 +20,14 @@ PROJECT_AUTHOR = "piehler73"
 
 
 # Lupusec API Calls: Requests and Headers
-URL_HTTP = "http://"
+URL_HTTP = "https://"
 URL_ACTION = "/action/"
 UPDATE_FREQ = 2
 
 LOGIN_REQUEST = "login"
 INFO_REQUEST = "welcomeGet"
 INFO_HEADER = "updates"
+SET_ALARM_REQUEST = "panelCondPost"
 DEVICE_LIST_REQUEST = "deviceListGet"
 DEVICE_LIST_HEADER = "senrows"
 HISTORY_REQUEST = "historyGet"
@@ -59,30 +60,34 @@ ALARM_DEVICE_ID = "0"
 ALARM_TYPE = "Alarm"
 
 
-# GENERIC Lupusec DEVICE TYPES
-TYPE_WINDOW = "Fensterkontakt"
-TYPE_DOOR = "Türkontakt"
-TYPE_CONTACT_XT2 = 4
+# Lupusec XT2 DEVICE TYPES
+# Single Device Types
+TYPE_BIN_SENSOR_XT2 = 4
 TYPE_WATER_XT2 = 5
 TYPE_SMOKE_XT2 = 11
-TYPE_POWER_SWITCH_1_XT2 = 24
-TYPE_POWER_SWITCH_2_XT2 = 25
-TYPE_POWER_SWITCH = "Steckdose"
-TYPE_SWITCH = [TYPE_POWER_SWITCH, TYPE_POWER_SWITCH_1_XT2, TYPE_POWER_SWITCH_2_XT2]
-TYPE_OPENING = [TYPE_DOOR, TYPE_WINDOW, TYPE_CONTACT_XT2]
-BINARY_SENSOR_TYPES = TYPE_OPENING
-TYPE_SENSOR = ["Rauchmelder", "Wassermelder", TYPE_WATER_XT2, TYPE_SMOKE_XT2]
+TYPE_SWITCH_INT_XT2 = 48
+TYPE_SWITCH_EXT_XT2 = 81
+TYPE_UPDOWN_SWITCH_XT2 = 76
+TYPE_THERMAL_SWITCH_XT2 = 79
+TYPE_LIGHT_XT2 = 74
+
+# Aggregated Device Types
+TYPES_BIN_SENSOR = [TYPE_BIN_SENSOR_XT2]
+TYPES_SENSOR = [TYPE_WATER_XT2, TYPE_SMOKE_XT2]
+TYPES_SWITCH = [TYPE_SWITCH_INT_XT2, TYPE_SWITCH_EXT_XT2]
+TYPES_UPDOWN_SWITCH = [TYPE_UPDOWN_SWITCH_XT2]
+TYPES_THERMAL_SWITCH = [TYPE_THERMAL_SWITCH_XT2]
+
+
+# Type Translations
 TYPE_TRANSLATION = {
     "Fensterkontakt": "window",
     "Türkontakt": "door",
-    TYPE_CONTACT_XT2: "Fenster-/Türkontakt",
+    TYPE_BIN_SENSOR_XT2: "Fenster-/Türkontakt",
     TYPE_WATER_XT2: "Wassermelder",
     TYPE_SMOKE_XT2: "Rauchmelder",
 }
 
-DEVICES_API_XT1 = "sensorListGet"
-DEVICES_API_XT2 = "deviceListGet"
-
 
 # Alarm Modes and Zones
 MODE_AWAY = "Arm"
